/*
 * Graphic lambda calculus in WebGL. This is a clone of the "GCalc" program
 * included in the "plt-games" package from the Racket Scheme distribution.
 * The original GCalc program was written by Eli Barzilay.
 *
 * Copyright (c) 2017 Pim Goossens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// GCalc expression associated with currently dragged object.
var dragexpr;

function clonethumbnail()
{
	var newthumbnail = document.createElement("canvas");

	newthumbnail.width = thumbnail.width;
	newthumbnail.height = thumbnail.height;
	var ctx = newthumbnail.getContext("2d");
	ctx.drawImage(thumbnail, 0, 0);

	return newthumbnail;
}

function dragelement(e)
{
	var id = e.target.id;
	if (id == null)
		return false;

	var element = document.getElementById(id);
	if (element == null)
		return false;

	if (id == "main")
		dragexpr = grid.main;
	else if (element.className == "lcolor" || element.className == "rcolor")
		dragexpr = palette[id][1];
	else if (element.className == "memcell")
		dragexpr = mem[memcells[id]][0];

	if (dragexpr == null)
		return false;

	e.dataTransfer.setData("text/plain", id);

	return true;
}

function canvasdrag(e)
{
	if (!dragelement(e))
		return;

	var newthumbnail = clonethumbnail();
	e.dataTransfer.setDragImage(newthumbnail, 0, 0);
}

function canvasdrop(e)
{
	if (!dragexpr)
		return;

	var x = e.clientX - canvasleft;
	var y = e.clientY - canvastop;
	var cw = canvaswidth;
	var ch = canvasheight;

	if      (inarea(x, y,     0, .3*ch, .3*cw, .7*ch))
		grid.left(dragexpr);
	else if (inarea(x, y, .7*cw, .3*ch,    cw, .7*ch))
		grid.right(dragexpr);
	else if (inarea(x, y, .3*cw,     0, .7*cw, .3*ch))
		grid.over(dragexpr);
	else if (inarea(x, y, .3*cw, .7*ch, .7*cw,    ch))
		grid.under(dragexpr);
	else if (inarea(x, y,     0,     0, .3*cw, .3*ch))
		grid.back(dragexpr);
	else if (inarea(x, y, .7*cw, .7*ch,    cw,    ch))
		grid.front(dragexpr);
	else if (inarea(x, y, .7*cw,     0,    cw, .3*ch))
		grid.applyf(dragexpr);
	else if (inarea(x, y,     0, .7*ch, .3*cw,    ch))
		grid.applya(dragexpr);
	else if (inarea(x, y, .3*cw, .3*ch, .7*cw, .7*ch))
		grid.set(dragexpr);

	grid.eval();

	draw();
}

function memorydrop(e)
{
	var index = memcells[e.target.id];
	if (!(index >= 0 && index < MEMGRID_SIZE))
		return;

	console.log("stored " + dragexpr + " in memory cell " + index);
	mem[index][0] = dragexpr;

	var cellcanvas = document.getElementById(mem[index][1]);
	var ctx = cellcanvas.getContext("2d");
	ctx.drawImage(clonethumbnail(), 0, 0, cellcanvas.width, cellcanvas.height);
}

function dropelement(e)
{
	if (!dragexpr)
		return;

	e.preventDefault();

	if (e.target == elements.main)
		canvasdrop(e);
	else if (e.target.className == "memcell")
		memorydrop(e);
	else
		console.log("drop to " + e.target.id + " of " + dragexpr);

	dragexpr = undefined;
}

function checkdrop(e)
{
	var classname = e.target.className;

	if (!dragexpr)
		return;

	if (classname == "lcolor" ||
		classname == "rcolor" ||
		classname == "memcell" ||
		e.target.id == "main")
	{
		e.preventDefault();
	}
}

function initdragdrop()
{
	for (let child of elements.palette.children)
	{
		child.draggable = true;
		child.addEventListener("dragstart", dragelement);
	}

	for (let child of elements.memory.children)
	{
		child.draggable = true;
		child.addEventListener("dragstart", dragelement);
	}

	document.addEventListener("dragover", checkdrop);
	document.addEventListener("drop", dropelement);
}
