/*
 * Graphic lambda calculus in WebGL. This is a clone of the "GCalc" program
 * included in the "plt-games" package from the Racket Scheme distribution.
 * The original GCalc program was written by Eli Barzilay.
 *
 * Copyright (c) 2017 Pim Goossens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

"use strict";

const ELEMENT_NAMES = [
	"palette",
	"main",
	"memory",
	"gcalcbox",
	"vshader",
	"vserr",
	"fshader",
	"fserr",
	"perr",
	"avshader",
	"avserr",
	"afshader",
	"afserr",
	"aperr",
	"stroke"
];

const elements = {};

var canvasleft;
var canvastop;
var canvaswidth;
var canvasheight;
var canvasthumbnail;

var cellwidth;
var cellheight;
var thumbnail;

function getpolymode()
{
	if (elements.stroke.checked)
		return gl.LINE_LOOP;
	else
		return gl.TRIANGLES;
}

function getmainsize()
{
	var style = window.getComputedStyle(elements.gcalcbox, null);
	var width = parseInt(style.getPropertyValue("width"));
	var height = parseInt(style.getPropertyValue("height"));
	return [width, height];
}

function resized()
{
	[elements.main.width, elements.main.height] = getmainsize();

	canvasleft = elements.main.offsetLeft;
	canvastop = elements.main.offsetTop;
	canvaswidth = elements.main.width;
	canvasheight = elements.main.height;

	// Determing cell size by computing style of first memory cell.
	var memcell = document.getElementById(mem[0][1]);
	var memcs = window.getComputedStyle(memcell);
	cellwidth = parseInt(memcs.width);
	cellheight = parseInt(memcs.height);

	thumbnail = document.createElement("canvas");
	thumbnail.width = cellwidth;
	thumbnail.height = cellheight;

	//console.log(cellwidth, cellheight);

	gl.viewport(0, 0, canvaswidth, canvasheight);
	draw();
}

function initelements()
{
	for (let id of ELEMENT_NAMES)
		elements[id] = document.getElementById(id);
}
