/*
 * Graphic lambda calculus in WebGL. This is a clone of the "GCalc" program
 * included in the "plt-games" package from the Racket Scheme distribution.
 * The original GCalc program was written by Eli Barzilay.
 *
 * Copyright (c) 2017 Pim Goossens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

"use strict";

const INDEX_ID = 0;
const INDEX_TEXT = 1;
const INDEX_RGB = 2;

const COLORS =
[
	/*     ID             RGB            Text     */
	["transparent", "Transparent",            null],
	["black",       "Black",        [  0,   0,   0]],
	["darkgray",    "Dark gray",    [ 64,  64,  64]],
	["gray",        "Gray",         [128, 128, 128]],
	["lightgray",   "Light gray",   [192, 192, 192]],
	["white",       "White",        [255, 255, 255]],
	["darkred",     "Dark red",     [128,   0,   0]],
	["red",         "Red",          [255,   0,   0]],
	["darkgreen",   "Dark green",   [  0, 128,   0]],
	["green",       "Green",        [  0, 255,   0]],
	["darkblue",    "Dark blue",    [  0,   0, 128]],
	["blue",        "Blue",         [  0,   0, 255]],
	["darkyellow",  "Dark yellow",  [128, 128,   0]],
	["yellow",      "Yellow",       [255, 255,   0]],
	["darkcyan",    "Dark cyan",    [  0, 128, 128]],
	["cyan",        "Cyan",         [  0, 255, 255]],
	["darkmagenta", "Dark magenta", [128,   0, 128]],
	["magenta",     "Magenta",      [255,   0, 255]]
];

var palette = {};

/* 0-256 */
const HEXDIGITS = "0123456789abcdef";
function hex(n)
{
	return HEXDIGITS[(n >> 4) & 0x0f] + HEXDIGITS[n & 0x0f];
}

function rgb2hex(rgb)
{
	if (rgb == null || rgb.length != 3)
		return null;

	return "#" + hex(rgb[0]) + hex(rgb[1]) + hex(rgb[2]);
}

function initpalette()
{
	for (var i=0; i<COLORS.length; i++)
	{
		var color = COLORS[i];
		var element = document.createElement("div");

		element.id = "color-" + color[INDEX_ID];
		if ((elements.palette.children.length % 2) == 0)
			element.className = "lcolor";
		else
			element.className = "rcolor";

		var rgb = color[INDEX_RGB];
		if (rgb != null)
			element.style.background = rgb2hex(rgb);

		element.innerHTML = color[INDEX_TEXT];

		palette[element.id] = [i, mkrgb(rgb)];

		/* *** Hack Alert! ***
		 * So... apparently variables declared within a loop aren't
		 * local to a single iteration, but are instead scoped at the
		 * same level as "i" and recycled, requiring us to explicitly
		 * introduce a new binding with its current value. What the
		 * f***, JavaScript?
		 */
		element.addEventListener("mousedown", function(rgb)
		{
			return function(e)
			{
				if (e.button == 2)
				{
					grid.lambda(rgb);
					draw();
				}
			};
		}(rgb));
		element.addEventListener("contextmenu", e => e.preventDefault());

		elements.palette.appendChild(element);
	}
}
